# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import random
import string

from django.conf import settings
from django.contrib.auth.models import AnonymousUser, User
from django.contrib.sites.models import Site
from django.test import RequestFactory, TransactionTestCase
from django.utils.translation import override

from cms import api
from cms.models import Title
from cms.utils import get_cms_setting
from cms.utils.i18n import get_language_list
from djangocms_helper.utils import create_user
from parler.utils.context import switch_language

from test_addon.models import Simple, Untranslated


class TestUtilityMixin(object):
    @staticmethod
    def reload(obj, language=None):
        """
        Simple convenience method for re-fetching an object from the ORM,
        optionally "as" a specified language.
        """
        try:
            new_obj = obj.__class__.objects.language(language).get(id=obj.id)
        except:
            new_obj = obj.__class__.objects.get(id=obj.id)
        return new_obj

    """Just adds some common test utilities to the testing class."""
    @staticmethod
    def rand_str(prefix='', length=16, chars=string.ascii_letters):
        return prefix + ''.join(random.choice(chars) for _ in range(length))

    def assertEqualItems(self, a, b):
        try:
            # In Python3, this method has been renamed (poorly)
            return self.assertCountEqual(a, b)
        except:
            # In 2.6, assertItemsEqual() doesn't sort first
            return self.assertItemsEqual(sorted(a), sorted(b))


class SimpleTestMixin(TestUtilityMixin):
    """This mixin adds data and methods useful for testing Things."""
    data = {
        'simple1': {
            'de': {'name': 'Objekt eins', },
            'en': {'name': 'Simple one', },
            'fr': {'name': 'objet un'},
        },
        'simple2': {
            'de': {'name': 'Objekt zwei', },
            'en': {'name': 'Simple two', },
            'fr': {'name': 'objet deux', },
        },
        'untranslated1': {
            'name': 'Untranslated Object',
            'slug': 'untranslated-object',
        }
    }

    simple1 = None
    simple2 = None
    untranslated1 = None

    def setUp(self):
        super(SimpleTestMixin, self).setUp()
        with override('de'):
            self.simple1 = Simple()
            self.simple1.name = self.data['simple1']['de']['name']
            self.simple1.save()

            self.simple2 = Simple()
            self.simple2.name = self.data['simple2']['de']['name']
            self.simple2.save()

        for lc in ['en', 'fr']:
            with switch_language(self.simple1, lc):
                self.simple1.name = self.data['simple1'][lc]['name']
                self.simple1.save()
            with switch_language(self.simple2, lc):
                self.simple2.name = self.data['simple2'][lc]['name']
                self.simple2.save()

        if not self.untranslated1:
            self.untranslated1 = Untranslated(**self.data['untranslated1'])
            self.untranslated1.save()


class CMSRequestBasedTest(TestUtilityMixin, TransactionTestCase):
    """Sets-up User(s) and CMS Pages for testing."""
    languages = get_language_list()

    @classmethod
    def setUpClass(cls):
        cls.request_factory = RequestFactory()
        if not User.objects.filter(username='normal').count():
            cls.user = create_user('normal', 'normal@admin.com', 'normal')
        cls.site1 = Site.objects.get(pk=1)

    @classmethod
    def tearDownClass(cls):
        User.objects.all().delete()

    def setUp(self):
        super(CMSRequestBasedTest, self).setUp()
        self.template = get_cms_setting('TEMPLATES')[0][0]
        self.language = settings.LANGUAGES[0][0]
        self.root_page = api.create_page(
            'root page',
            self.template,
            self.language,
            published=True,
        )
        self.page = api.create_page(
            'Simple Page',
            self.template,
            self.language,
            published=True,
            parent=self.root_page,
            apphook='SimpleApp',
            apphook_namespace='simple',
        )

        self.placeholder = self.page.placeholders.all()[0]
        self.request = self.get_request('en')

        for page in [self.root_page, self.page]:
            for language, _ in settings.LANGUAGES[1:]:
                api.create_title(language, page.get_slug(), page)
                page.publish(language)

    @classmethod
    def get_request(cls, language=None, url="/"):
        """
        Returns a Request instance populated with cms specific attributes.
        """
        request = cls.request_factory.get(url)
        request.session = {}
        request.LANGUAGE_CODE = language or settings.LANGUAGE_CODE
        # Needed for plugin rendering.
        request.current_page = None
        request.user = AnonymousUser()
        return request

    def get_or_create_page(self, base_title=None, languages=None):
        """Creates a page with a given title, or, if it already exists, just
        retrieves and returns it."""
        from cms.api import create_page, create_title
        if not base_title:
            # No title? Create one.
            base_title = self.rand_str(prefix="page", length=8)
        if not languages:
            # If no langs supplied, use'em all
            languages = self.languages
        # If there is already a page with this title, just return it.
        try:
            page_title = Title.objects.get(title=base_title)
            return page_title.page.get_draft_object()
        except:
            pass

        # No? Okay, create one.
        page = create_page(base_title, 'fullwidth.html', language=languages[0])
        # If there are multiple languages, create the translations
        if len(languages) > 1:
            for lang in languages[1:]:
                title_lang = "{0}-{1}".format(base_title, lang)
                create_title(language=lang, title=title_lang, page=page)
                page.publish(lang)
        return page.get_draft_object()

    def get_page_request(
            self, page, user, path=None, edit=False, lang_code='en'):
        from cms.middleware.toolbar import ToolbarMiddleware
        path = path or page and page.get_absolute_url()
        if edit:
            path += '?edit'
        request = RequestFactory().get(path)
        request.session = {}
        request.user = user
        request.LANGUAGE_CODE = lang_code
        if edit:
            request.GET = {'edit': None}
        else:
            request.GET = {'edit_off': None}
        request.current_page = page
        mid = ToolbarMiddleware()
        mid.process_request(request)
        return request


class SimpleTransactionTestCase(SimpleTestMixin, CMSRequestBasedTest):
    pass



# -*- coding: utf-8 -*

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool


class SimpleApp(CMSApp):
    name = _('Simple')
    urls = ['test_addon.urls']

apphook_pool.register(SimpleApp)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from slugify import slugify

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
try:
    from django.utils.encoding import force_text
except ImportError:  # pragma: no cover
    from django.utils.encoding import force_unicode as force_text
from django.utils.translation import ugettext_lazy as _

from cms.utils.i18n import (
    get_current_language,
    get_default_language,
    get_fallback_languages,
)


class TranslatedAutoSlugifyMixin(object):
    """
    This is a TranslatableModel mixin that automatically generates a suitable
    slug for the object on save.

    If `slug_globally_unique` is True, then slugs will be required to be
    unique across all languages.

    If `slug_globally_unique` is False (default), then the strategy used here
    is that it is OK for two objects to use the same slug if the slugs are for
    different languages. So if this were used on an Article model, these would
    be valid:

        /en/pain -> Article in EN about physical discomfort
        /fr/pain -> Article in FR about bread

    Of course, this means that when resolving an object from its URL, care must
    be taken to factor in the language segment of the URL too.
    """

    # Default slug to use if the current object produces an empty slug. A
    # default is assembled via introspection. Implementing class should provide
    # a lazy translated string here.
    slug_default = None
    # The translated slug field name.
    slug_field_name = 'slug'
    # Flag controlling if slugs are unique per language or globally unique.
    slug_globally_unique = False
    # Max length of the slug. By default is determined by introspection.
    slug_max_length = None
    # The separator to use before any index.
    slug_separator = '-'
    # The translated field to derive a slug from. If `get_slug_source()` is
    # overridden in the model, overriding `get_slug_default` is recommended.
    slug_source_field_name = None
    # filters that would be used to determine slug uniqueness, would be
    # populated with slug_field_name.
    raw_slug_filter_string = 'translations__{0}'

    # python-slugify option for smart truncate
    word_boundary = False
    # python-slugify option for smart truncate
    save_order = True

    def get_slug_default(self):
        """
        Naively constructs a translated default slug from the object. For
        better results, just set the `slug_default` property on the class to a
        lazy translated string. Alternatively, override this method if you need
        to more programmatically determine the default slug.

        Example: If your model is "news article" and your source field is
        "title" this will return "news-article-without-title".
        """
        if self.slug_default:
            # Implementing class provides its own, translated string, use it.
            return force_text(self.slug_default)

        object_name = self._meta.verbose_name

        # Introspect the field name
        try:
            trans_meta = self.translations.model._meta
            source_field = trans_meta.get_field(
                self.slug_source_field_name)
            field_name = getattr(source_field, 'verbose_name')
        except:
            field_name = _('name')

        slug_default = _("{0}-without-{1}").format(
            slugify(force_text(object_name)),
            slugify(force_text(field_name)),
        )
        return slug_default

    def get_slug_max_length(self, idx_len=0):
        """
        Introspects the slug field to determine the max length, taking into
        account a possible separator and up to a [idx_len]-digit number.
        """
        if self.slug_max_length:
            slug_max_length = self.slug_max_length
        else:
            trans_meta = self.translations.model._meta
            slug_field = trans_meta.get_field(self.slug_field_name)
            max_length = getattr(slug_field, 'max_length', 255)
            # All objects of this class will use the same value.
            slug_max_length = max_length
        if idx_len:
            return slug_max_length - len(self.slug_separator) - idx_len
        return slug_max_length

    def get_slug_source(self):
        """
        Simply returns the value of the slug source field. Override for more
        complex situations such as using multiple fields as the source.
        """
        return getattr(self, self.slug_source_field_name, None)

    def _get_candidate_slug(self, slug, idx=0):
        return "{slug}{sep}{idx}".format(
            slug=slug, sep=self.slug_separator, idx=idx)

    def _get_existing_slug(self):
        """
        Get slug if it is set on object, slug is taken from
        `self.slug_field_name` field.
        """
        return getattr(self, self.slug_field_name, None)

    def slugify(self, text, max_length=None):
        """
        Generate slug using appropriate slugification utility.
        """
        if max_length is None:
            max_length = self.get_slug_max_length()
        slug = slugify(text,
                       max_length=max_length,
                       word_boundary=self.word_boundary,
                       save_order=self.save_order,
                       separator=self.slug_separator)
        return slug

    def _get_ideal_slug(self):
        """Build the "ideal slug" for this object as a starting point"""
        source = self.get_slug_source()
        if source:
            source = force_text(source)
            ideal_slug = force_text(self.slugify(source))
        else:
            # For some reason, the slug came back empty, use the default
            ideal_slug = force_text(self.get_slug_default())

        # Trim the length of the ideal slug to the limit allowed the field
        max_length = self.get_slug_max_length()
        return ideal_slug[:max_length]

    def _get_slug_queryset(self, lookup_model=None):
        """
        Build the queryset we will be using considering options and the
        object's state.
        lookup model - model manager to build base queryset. If none
        self.__class__ would be used.
        """
        language = self.get_current_language() or get_default_language()
        if lookup_model is None:
            lookup_model = self.__class__

        qs = lookup_model.objects.language(language)
        if not self.slug_globally_unique:
            qs = qs.filter(
                translations__language_code=language)
        if self.pk:
            qs = qs.exclude(pk=self.pk)
        return qs

    def _slug_exists(self, slug, slug_filter=None, qs=None):
        """
        Check if slug exists in the given queryset.
        If slug_filter is None it would be created from
        self.raw_slug_filter_string and self.slug_field_name
        """

        if qs is None:
            qs = self._get_slug_queryset()
        if slug_filter is None:
            slug_filter = self.raw_slug_filter_string.format(
                self.slug_field_name)
        return qs.filter(**{slug_filter: slug}).exists()

    def make_new_slug(self, slug=None, qs=None):
        """
        Generate a slug that meets requirements.
        :param qs: queryset to check uniqueness,
                   if None - self._get_slug_queryset() queryset will be used
        :param slug: candidate slug, new slug would be generated from this
                     value, if None - self._get_ideal_slug() would be used
        :return (str): slug
        """

        if not slug:
            # Build the "ideal slug" for this object as a starting point
            slug = self._get_ideal_slug()
        # initial setup
        idx = 1
        candidate = slug
        max_length = self.get_slug_max_length()
        # Check if the resulting slug is currently in use, if not, use it.
        # Otherwise, add a separator and an index until we find an
        # unused combination.
        while self._slug_exists(candidate, qs=qs):
            if len(candidate) > max_length:
                max_length = self.get_slug_max_length(len(str(idx)))
            candidate = self._get_candidate_slug(slug[:max_length], idx)
            idx += 1
        return candidate

    def save(self, **kwargs):
        slug = self._get_existing_slug()
        if not slug or self._slug_exists(slug):
            slug = self.make_new_slug(slug=slug)
            setattr(self, self.slug_field_name, slug)
        return super(TranslatedAutoSlugifyMixin, self).save(**kwargs)


class TranslationHelperMixin(object):

    def known_translation_getter(self, field, default=None,
                                 language_code=None, any_language=False):
        """
        This is meant to act like HVAD/Parler's safe_translation_getter() but
        respects the fallback preferences as defined in
        `settings.CMS_LANGUAGES` and returns both the translated field value
        and the language it represents as a tuple: (value, language).

        If no suitable language is found, then it returns (default, None)
        """
        # NOTE: We're using the CMS fallbacks here, rather than the Parler
        # fallbacks, the developer should ensure that their project's Parler
        # settings match the CMS settings.
        try:
            object_languages = self.get_available_languages()
            assert hasattr(object_languages, '__iter__')
        except [KeyError, AssertionError]:
            raise ImproperlyConfigured(
                "TranslationHelperMixin must be used with a model defining"
                "get_available_languages() that returns a list of available"
                "language codes. E.g., django-parler's TranslatableModel.")

        language_code = (
            language_code or get_current_language() or get_default_language())
        site_id = getattr(settings, 'SITE_ID', None)
        languages = [language_code] + get_fallback_languages(
            language_code, site_id=site_id)

        # Grab the first language that is common to our list of fallbacks and
        # the list of available languages for this object.
        if languages and object_languages:
            language_code = next(
                (lang for lang in languages if lang in object_languages), None)

            if language_code:
                value = self.safe_translation_getter(field,
                    default=default, language_code=language_code)
                return (value, language_code)

        # No suitable translation exists
        return (default, None)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _, override, get_language

from aldryn_translation_tools.models import (
    TranslatedAutoSlugifyMixin,
    TranslationHelperMixin,
)
from parler.models import TranslatableModel, TranslatedFields

from .managers import SimpleManager


@python_2_unicode_compatible
class Simple(TranslatedAutoSlugifyMixin, TranslationHelperMixin,
             TranslatableModel):
    slug_source_field_name = 'name'

    translations = TranslatedFields(
        name=models.CharField(max_length=64),
        slug=models.SlugField(max_length=64, blank=True, default='')
    )

    objects = SimpleManager()

    def get_absolute_url(self, language=None):
        language = language or get_language()
        slug, language = self.known_translation_getter(
            'slug', None, language_code=language)
        kwargs = {'slug': slug}

        with override(language):
            return reverse('simple:simple-detail', kwargs=kwargs)

    def __str__(self):
        return self.safe_translation_getter(
            'name', default="Simple: {0}".format(self.pk))


@python_2_unicode_compatible
class Untranslated(models.Model):
    name = models.CharField(max_length=64)
    slug = models.SlugField(max_length=64, blank=True, default='')

    def get_absolute_url(self):
        return reverse(
            'simple:untranslated-detail', kwargs={'slug': self.slug})

    def __str__(self):
        return "Untranslated: {0}".format(self.name)


@python_2_unicode_compatible
class Unconventional(TranslatedAutoSlugifyMixin, TranslatableModel):
    slug_source_field_name = 'title'
    slug_field_name = 'unique_slug'

    translations = TranslatedFields(
        title=models.CharField(_('short title'), max_length=64),
        unique_slug=models.SlugField(max_length=64, blank=True, default='')
    )

    class Meta:
        verbose_name = _('unconventional model')

    def __str__(self):
        return self.safe_translation_getter(
            'title', default="Unconventional: {0}".format(self.pk))


class Complex(TranslatedAutoSlugifyMixin, TranslatableModel):

    translations = TranslatedFields(
        name=models.CharField(max_length=64),
        slug=models.SlugField(max_length=64, blank=True, default='')
    )

    object_type = models.CharField(max_length=64)

    def get_slug_source(self):
        if self.object_type and self.name:
            return "{type}: {name}".format(
                type=self.object_type,
                name=self.safe_translation_getter('name', default='unnamed')
            )
        else:
            return None

    def __str__(self):
        return self.get_slug_source()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.forms import widgets
from django.utils.translation import force_text, ugettext as _

from cms.utils.i18n import get_current_language
from cms.utils.urlutils import admin_reverse


class LinkedRelatedInlineMixin(object):
    """
    This InlineAdmin mixin links the first field to the row object's own admin
    change form.

    NOTE: If the first field is editable, it is undefined what will happen.
    For best results, consider making all fields readonly (since they can be
    edited with ease by following the link), and disabling the ability to add
    new objects by overriding has_add_permission() on the inline to always
    return false.
    """

    extra = 0

    class ReverseLink:

        allow_tags = True

        def __init__(self, display_link="link"):
            self.display_link = display_link
            self.short_description = display_link

        def __call__(self, obj):
            model_name = obj.__class__.__name__.lower()
            admin_link = admin_reverse(
                "{app_label}_{model_name}_change".format(
                    app_label=obj._meta.app_label.lower(),
                    model_name=model_name,
                ), args=(obj.id, ))
            return '<a href="{admin_link}" title="{title}">{link}</a>'.format(
                admin_link=admin_link,
                title=_('Click to view or edit this {0}').format(
                    obj._meta.verbose_name),
                link=getattr(obj, self.display_link))

    def __init__(self, parent_model, admin_site):
        self.original_fields = self.get_fields_list(None)
        if len(self.original_fields):
            self.fields = ["reverse_link", ] + self.original_fields[1:]
        else:
            self.fields = ["reverse_link"]
        self.reverse_link = self.ReverseLink(self.original_fields[0])
        super(LinkedRelatedInlineMixin, self).__init__(
            parent_model, admin_site)

    def get_fields_list(self, request, obj=None):
        """
        Returns a list of the AdminModel's declared `fields`, or, constructs it
        from the object, then, removes any `exclude`d items.
        """
        # ModelAdmin.get_fields came in Django 1.7, I believe
        if hasattr(super(LinkedRelatedInlineMixin, self), "get_fields"):
            fields = super(
                LinkedRelatedInlineMixin, self).get_fields(request, obj)
        elif self.fields:
            fields = self.fields
        else:
            fields = [f.name for f in self.model._meta.local_fields]
        if fields and self.exclude:
            fields = [f for f in fields if f not in self.exclude]
        if fields:
            return list(fields)
        else:
            return []

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super(
            LinkedRelatedInlineMixin, self).get_readonly_fields(request, obj)
        if "reverse_link" not in readonly_fields:
            readonly_fields = list(readonly_fields) + ["reverse_link", ]
        # We want all fields to be readonly for this inline
        return readonly_fields


class AllTranslationsMixin(object):

    @property
    def media(self):
        return super(AllTranslationsMixin, self).media + widgets.Media(
            css={'all': ('css/admin/all-translations-mixin.css', ), }
        )

    def all_translations(self, obj):
        """
        Adds a property to the list_display that lists all translations with
        links directly to their change forms. Includes CSS to style the links
        to looks like tags with color indicating current language, active and
        inactive translations.

        A similar capability is in HVAD, and now there is this for
        Parler-based projects.
        """
        available = list(obj.get_available_languages())
        current = get_current_language()
        langs = []
        for code, lang_name in settings.LANGUAGES:
            classes = ["lang-code", ]
            title = force_text(lang_name)
            if code == current:
                classes += ["current", ]
            if code in available:
                classes += ["active", ]
                title += " (translated)"
            else:
                title += " (untranslated)"
            change_form_url = admin_reverse(
                '{app_label}_{model_name}_change'.format(
                    app_label=obj._meta.app_label.lower(),
                    model_name=obj.__class__.__name__.lower(),
                ), args=(obj.id, )
            )
            link = ('<a class="{classes}" href="{url}?language={code}" ' +
                'title="{title}">{code}</a>').format(
                classes=' '.join(classes),
                url=change_form_url,
                code=code,
                title=title,
            )
            langs.append(link)
        return ''.join(langs)
    all_translations.short_description = 'Translations'
    all_translations.allow_tags = True

    def get_list_display(self, request):
        """
        Unless the the developer has already placed "all_translations" in the
        list_display list (presumably specifically where she wants it), append
        the list of translations to the end.
        """
        list_display = super(
            AllTranslationsMixin, self).get_list_display(request)
        if 'all_translations' not in list_display:
            list_display = list(list_display) + ['all_translations', ]
        return list_display



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

try:
    from urllib import urlencode
except ImportError:  # pragma: no cover
    from urllib.parse import urlencode

from django.utils.translation import get_language_from_request

from cms.utils.urlutils import admin_reverse
from parler.models import TranslatableModel


def get_admin_url(action, action_args=[], **url_args):
    """
    Convenience method for constructing admin-urls with GET parameters.

    :param action:      The admin url key for use in reverse. E.g.,
                        'things_edit_thing'
    :param action_args: The url args for the reverse. E.g., [thing.pk, ]
    :param url_args:    A dict of key/value pairs for GET parameters. E.g.,
                        {'language': 'en', }.
    :return: The complete admin url
    """
    base_url = admin_reverse(action, args=action_args)
    # Converts [{key: value}, …] => ["key=value", …]
    # We sort the dict into a sequence of tuples for predictability. Testing
    # reveals that different versions of Python/Django behave differently, which
    # make it complicated to run tests, if nothing else.
    url_args_tuple = sorted([(k, v) for k, v in url_args.items()])
    params = urlencode(url_args_tuple)
    if params:
        return "?".join([base_url, params])
    else:
        return base_url


def get_object_from_request(model, request,
                            pk_url_kwarg='pk',
                            slug_url_kwarg='slug',
                            slug_field='slug'):
    """
    Given a model and the request, try to extract and return an object
    from an available 'pk' or 'slug', or return None.

    Note that no checking is done that the obj's kwargs really are for objects
    matching the provided model (how would it?) so use only where appropriate.
    """
    language = get_language_from_request(request, check_path=True)
    kwargs = request.resolver_match.kwargs
    mgr = model.objects
    if pk_url_kwarg in kwargs:
        return mgr.filter(pk=kwargs[pk_url_kwarg]).first()
    elif slug_url_kwarg in kwargs:
        # If the model is translatable, and the given slug is a translated
        # field, then find it the Parler way.
        filter_kwargs = {slug_field: kwargs[slug_url_kwarg]}
        try:
            translated_fields = model._parler_meta.get_translated_fields()
        except AttributeError:
            translated_fields = []
        if (issubclass(model, TranslatableModel) and
                slug_url_kwarg in translated_fields):
            return mgr.active_translations(language, **filter_kwargs).first()
        else:
            # OK, do it the normal way.
            return mgr.filter(**filter_kwargs).first()
    else:
        return None



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.test import TransactionTestCase
from django.utils.translation import ugettext_lazy as _

from test_addon.models import Complex, Simple, Unconventional


class TestTranslatableAutoSlugifyMixin(TransactionTestCase):

    def test_simple_slug(self):
        simple = Simple()
        simple.set_current_language('en')
        simple.name = 'Simple'
        simple.save()
        self.assertEquals(simple.slug, 'simple')

    def test_unconventional_slug(self):
        unconventional = Unconventional()
        unconventional.set_current_language('en')
        unconventional.title = 'Unconventional'
        unconventional.save()
        self.assertEquals('unconventional', unconventional.unique_slug)

    def test_complex_slug(self):
        complex1 = Complex()
        complex1.set_current_language('en')
        complex1.name = 'one'
        complex1.object_type = 'complex'
        complex1.save()
        self.assertEquals('complex-one', complex1.slug)

    def test_existing_object(self):
        simple = Simple()
        simple.set_current_language('en')
        simple.save()
        # slug is now the default
        simple.name = 'A new name'
        simple.slug = None
        simple.save()
        self.assertEquals('a-new-name', simple.slug)

    def test_limited_length(self):
        Simple.slug_max_length = 6
        try:
            for r in range(0, 101):
                simple = Simple()
                simple.set_current_language('en')
                simple.name = 'Simple'
                simple.save()
        except:
            self.fail()
        Simple.slug_max_length = None

    def test_slug_unique_global(self):
        Simple.slug_globally_unique = True

        simple_en = Simple()
        simple_en.set_current_language('en')
        simple_en.name = 'SimpleOne'
        simple_en.save()

        simple_fr = Simple()
        simple_fr.set_current_language('fr')
        simple_fr.name = 'SimpleOne'
        simple_fr.save()

        self.assertNotEquals(simple_en.slug, simple_fr.slug)

        Simple.slug_globally_unique = None  # default is False

        simple_en = Simple()
        simple_en.set_current_language('en')
        simple_en.name = 'SimpleTwo'
        simple_en.save()

        simple_fr = Simple()
        simple_fr.set_current_language('fr')
        simple_fr.name = 'SimpleTwo'
        simple_fr.save()

        self.assertEquals(simple_en.slug, simple_fr.slug)

    def test_slug_unique_for_language(self):
        simple_en_1 = Simple()
        simple_en_1.set_current_language('en')
        simple_en_1.name = 'SimpleOne'
        simple_en_1.save()
        # make another instance with same name
        simple_en_2 = Simple()
        simple_en_2.set_current_language('en')
        simple_en_2.name = 'SimpleOne'
        simple_en_2.save()
        # slugs should not be same.
        self.assertNotEquals(simple_en_1.slug, simple_en_2.slug)

    def test_slug_unique_for_language_if_slug_is_the_same(self):
        simple_en_1 = Simple()
        simple_en_1.set_current_language('en')
        simple_en_1.name = 'SimpleOne'
        simple_en_1.slug = 'simpleone'
        simple_en_1.save()
        # make another instance with same name
        simple_en_2 = Simple()
        simple_en_2.set_current_language('en')
        simple_en_2.name = 'SimpleOne'
        simple_en_2.slug = 'simpleone'
        simple_en_2.save()
        # slugs should not be same.
        self.assertNotEquals(simple_en_1.slug, simple_en_2.slug)

    def test_simple_slug_default(self):
        # First test that the default works
        simple = Simple()
        simple.set_current_language('en')
        simple.save()
        self.assertEquals(
            'simple-without-name', simple.get_slug_default())
        # Also test without explicit language
        self.assertEquals(
            'simple-without-name', simple.get_slug_default())

        # Now test that a default would be used if available
        Simple.slug_default = _('unnamed-simple-object')
        simple = Simple()
        simple.set_current_language('en')
        simple.save()
        self.assertEquals(
            'unnamed-simple-object', simple.get_slug_default())
        # Also test without explicit language
        self.assertEquals(
            'unnamed-simple-object', simple.get_slug_default())

    def test_unconventional_slug_default(self):
        unconventional = Unconventional()
        unconventional.set_current_language('en')
        unconventional.save()
        self.assertEquals(
            'unconventional-model-without-short-title',
            unconventional.get_slug_default()
        )

    def test_complex_slug_default(self):
        complex1 = Complex()
        complex1.set_current_language('en')
        complex1.save()
        self.assertEquals('complex-without-name', complex1.slug)
